@@ -1,4 +1,4 @@
-# Copyright (C) 2002-2005 by the Free Software Foundation, Inc.
+# Copyright (C) 2002-2008 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 """
     subscribe [password] [digest|nodigest] [address=<address>]
@@ -53,7 +53,7 @@ def process(res, args):
     # Parse the args
     argnum = 0
     for arg in args:
-        if arg.startswith('address='):
+        if arg.lower().startswith('address='):
             address = arg[8:]
         elif argnum == 0:
             password = arg
@@ -70,6 +70,13 @@ def process(res, args):
             res.results.append(gethelp(mlist))
             return STOP
         argnum += 1
+    # Fix the password/digest issue
+    if digest is None and password.lower() in ('digest', 'nodigest'):
+        if password.lower() == 'digest':
+            digest = 1
+        else:
+            digest = 0
+        password = None
     # Fill in empty defaults
     if digest is None:
         digest = mlist.digest_is_default
